from colored import fg

from pou.core.terminal import Terminal

from pou.pages.settings import Settings
from pou.pages.chatgpt import ChatGPT

LINE_UP = '\033[1A'
LINE_CLEAR = '\x1b[2K'


class Pou():
    def __init__(self):
        self.terminal = Terminal()
        self.quit = False

    def chatgpt(self):
        ChatGPT(self.terminal).start()
        pass

    def settings(self):
        Settings(self.terminal).start()
        pass

    def exit(self):
        self.quit = True
        self.terminal.box_text(f"See you {fg(self.terminal.HIGHLIGHT_COLOR)}soon!{fg(self.terminal.TEXT_COLOR)}")
        pass

    def configure_user_cred(self):
        ChatGPT(self.terminal).configure_user_cred()

    def run(self):
        user_input = self.terminal.selection(
            f"Welcome to {fg(self.terminal.HIGHLIGHT_COLOR)}pou",
            f"What would {fg(self.terminal.HIGHLIGHT_COLOR)}you{fg(self.terminal.TEXT_COLOR)} like to do today?",
            [f"{fg(self.terminal.TEXT_COLOR)}Back to Terminal",
             f"{fg(self.terminal.TEXT_COLOR)}Configure user credentials",
             f"{fg(self.terminal.TEXT_COLOR)}Settings", 
             f"{fg(self.terminal.TEXT_COLOR)}Exit"]
        )

        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Start": self.chatgpt()
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Configure user credentials": self.configure_user_cred()
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Settings": self.settings()
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Exit": self.exit()

    def start(self):
        self.chatgpt()
        while not self.quit:
            try:    
                self.run()
            except KeyboardInterrupt: self.exit()


import os
import json

# STORAGE KEYS
BOX_COLOR = "box_color"
HIGHLIGHT_COLOR = "highlight_color"
TEXT_COLOR = "text_color"

class Storage():
    @staticmethod
    def store(key : str, value : str):
        folder_path = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..', 'storage'))
        file_path = os.path.join(folder_path, 'config')

        if not os.path.exists(folder_path): os.mkdir(folder_path)
        
        data = {}
        if os.path.isfile(file_path): data = json.loads(open(file_path).read())
        data[key] = value

        open(file_path, 'w').write(json.dumps(data))

    @staticmethod
    def load(key : str):
        folder_path = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..', 'storage'))
        file_path = os.path.join(folder_path, 'config')

        if not os.path.exists(folder_path): os.mkdir(folder_path)
        
        if os.path.isfile(file_path): 
            data = json.loads(open(file_path).read())
            if key in data.keys(): return data[key]
            else: return None
        else: return None


from colored import fg
import re

from pou.core.terminal import Terminal
from pou.core.sleepy_colors import RED_COLOR, ORANGE_COLOR, YELLOW_COLOR, GREEN_COLOR, AQUAMARINE_COLOR, LAVENDER_COLOR, PURPLE_COLOR, WHITE_COLOR
from pou.core.storage import Storage, BOX_COLOR, TEXT_COLOR, HIGHLIGHT_COLOR

class Settings():

    def __init__(self, terminal: Terminal):
        self.terminal = terminal
        self.quit = False

    def is_color_hex_code(self, text : str):
        match = re.search(r'^#(?:[0-9a-fA-F]{3}){1,2}$', text)
        return match

    def color_custom(self):
        user_input = self.terminal.input("Custom color", "Input the color's hexcode (e.g #fff)")
        while not self.is_color_hex_code(user_input):
            user_input = self.terminal.input("Custom color", "Make sure to enter a valid color hexcode (e.g #fff)")
        
        return user_input

    def color(self, title : str):
        user_input = self.terminal.selection(
            title, 
            "What color would you like to change it to?", 
            [f"{fg(WHITE_COLOR)}Blank white{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(RED_COLOR)}Milk blood red{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(ORANGE_COLOR)}Citrus Juice Orange{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(YELLOW_COLOR)}Sunshine Yellow{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(GREEN_COLOR)}Fresh Vegetable Green{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(AQUAMARINE_COLOR)}Idunnowhat Blue{fg(self.terminal.TEXT_COLOR)}",   
             f"{fg(LAVENDER_COLOR)}Fragrant Lavender{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(PURPLE_COLOR)}Grape purple{fg(self.terminal.TEXT_COLOR)}",
             f"{fg(self.terminal.TEXT_COLOR)}Custom"             
            ]
        )

        if user_input == f"{fg(WHITE_COLOR)}Blank white{fg(self.terminal.TEXT_COLOR)}": return WHITE_COLOR
        if user_input == f"{fg(RED_COLOR)}Milk blood red{fg(self.terminal.TEXT_COLOR)}": return RED_COLOR
        if user_input == f"{fg(ORANGE_COLOR)}Citrus Juice Orange{fg(self.terminal.TEXT_COLOR)}": return ORANGE_COLOR
        if user_input == f"{fg(YELLOW_COLOR)}Sunshine Yellow{fg(self.terminal.TEXT_COLOR)}": return YELLOW_COLOR
        if user_input == f"{fg(GREEN_COLOR)}Fresh Vegetable Green{fg(self.terminal.TEXT_COLOR)}": return GREEN_COLOR
        if user_input == f"{fg(AQUAMARINE_COLOR)}Idunnowhat Blue{fg(self.terminal.TEXT_COLOR)}": return AQUAMARINE_COLOR
        if user_input == f"{fg(LAVENDER_COLOR)}Fragrant Lavender{fg(self.terminal.TEXT_COLOR)}": return LAVENDER_COLOR
        if user_input == f"{fg(PURPLE_COLOR)}Grape purple{fg(self.terminal.TEXT_COLOR)}": return PURPLE_COLOR
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Custom": return self.color_custom()
        return "#fff"
    
    def back(self):
        self.quit = True

    def run(self):
        user_input = self.terminal.selection(
            "Settings", 
            f"What would you like to {fg(self.terminal.HIGHLIGHT_COLOR)}change?{fg(self.terminal.TEXT_COLOR)}", 
            [f"{fg(self.terminal.TEXT_COLOR)}Text color", 
             f"{fg(self.terminal.TEXT_COLOR)}Box color", 
             f"{fg(self.terminal.TEXT_COLOR)}Highlight color", 
             f"{fg(self.terminal.TEXT_COLOR)}Back"]
        )

        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Text color": 
            self.terminal.TEXT_COLOR = self.color("Text color")
            Storage.store(TEXT_COLOR, self.terminal.TEXT_COLOR)
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Box color": 
            self.terminal.BOX_COLOR = self.color("Box color")
            Storage.store(BOX_COLOR, self.terminal.BOX_COLOR)
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Highlight color": 
            self.terminal.HIGHLIGHT_COLOR = self.color("Highlight color")
            Storage.store(HIGHLIGHT_COLOR, self.terminal.HIGHLIGHT_COLOR)
        if user_input == f"{fg(self.terminal.TEXT_COLOR)}Back": 
            self.back()

    def start(self):
        while not self.quit:
            self.run()



RED_COLOR = '#ed8796'
PINK_COLOR = '#ee99a0'
ORANGE_COLOR = '#f5a97f'
YELLOW_COLOR = '#eed49f'
GREEN_COLOR = '#a6da95'
AQUAMARINE_COLOR = '#8bd5ca'
LAVENDER_COLOR = '#b7bdf8'
PURPLE_COLOR = '#c6a0f6'
WHITE_COLOR = '#fff'
RESET = '\u001b[0m'


import os

from colored import fg
from boxing import boxing
import inquirer

from pou.core.sleepy_colors import YELLOW_COLOR, WHITE_COLOR, RESET
from pou.core.storage import Storage, BOX_COLOR, HIGHLIGHT_COLOR, TEXT_COLOR

class Terminal():

    def __init__(self):
        box_color = Storage.load(BOX_COLOR)
        highlight_color = Storage.load(HIGHLIGHT_COLOR)
        text_color = Storage.load(TEXT_COLOR)

        if box_color == None: box_color = YELLOW_COLOR
        if highlight_color == None: highlight_color = YELLOW_COLOR
        if text_color == None: text_color = WHITE_COLOR

        self.BOX_COLOR = box_color
        self.HIGHLIGHT_COLOR = highlight_color
        self.TEXT_COLOR = text_color

    def box_text(self, text : str):
        self.clear()
        self.print(boxing(text))

    def selection(self, title : str, question : str, selections):
        try:
            self.clear()
            self.print(boxing(title))

            questions = [inquirer.List('input', message=self.colorize_text(question), choices=selections, carousel=True)]
            user_input = inquirer.prompt(questions)["input"]
        except:
            raise KeyboardInterrupt
        
        return user_input
    
    def input(self, title : str, question : str):
        self.clear()

        self.print(
            boxing(title)
        )

        print(f"{question}\n> ", end="")
        return input()

    def colorize_text(self, text : str):
        return f"{fg(self.TEXT_COLOR)}{text}{fg(self.TEXT_COLOR)}"

    def print(self, text : str, end : str = "\n", sep : str = ""):
        to_replace = "┌─┐│└─┘"
        for char in to_replace:
            text = text.replace(char, f"{fg(self.BOX_COLOR)}{char}{fg(self.TEXT_COLOR)}")
        print(self.colorize_text(text), end=end, sep=sep)
        print(RESET, end="", sep="")

    def clear(self):
        os.system('cls' if os.name == 'nt' else 'clear')


import os
import json
import logging
import requests
import traceback

from colored import fg

from pou.core.terminal import Terminal
from pou.core.storage import Storage

CHATGPT_URL = "https://api.openai.com/v1/chat/completions"
CHATGPT_MODEL = "gpt-3.5-turbo"

ORGANIZATION_KEY = "organization_key"
API_KEY = "api_key"

class ChatGPT():
    def __init__(self, terminal: Terminal):
        self.terminal = terminal
        self.quit = False
        self.user_credentials = {ORGANIZATION_KEY: Storage.load(ORGANIZATION_KEY), API_KEY: Storage.load(API_KEY)}
    
    def shell(self):
        self.terminal.clear()
        if self.user_credentials[ORGANIZATION_KEY] == None or self.user_credentials[API_KEY] == None: self.configure_user_cred()
        
        self.terminal.box_text("Terminal")
        self.terminal.print(f"Write commands! (Type {fg(self.terminal.HIGHLIGHT_COLOR)}exit{fg(self.terminal.TEXT_COLOR)} to leave)")
        self.terminal.print(f"Available {fg(self.terminal.HIGHLIGHT_COLOR)}sleepychat{fg(self.terminal.TEXT_COLOR)} commands")
        self.terminal.print(f"- {fg(self.terminal.HIGHLIGHT_COLOR)}ask{fg(self.terminal.TEXT_COLOR)} - To ask ChatGPT questions. Example: {fg(self.terminal.HIGHLIGHT_COLOR)}ask 'Hello world'{fg(self.terminal.TEXT_COLOR)}")
        self.terminal.print(f"- {fg(self.terminal.HIGHLIGHT_COLOR)}translate{fg(self.terminal.TEXT_COLOR)} - Translate text to a particular language. Example: {fg(self.terminal.HIGHLIGHT_COLOR)}translate english 'hello world'{fg(self.terminal.TEXT_COLOR)}")
        self.terminal.print(f"- {fg(self.terminal.HIGHLIGHT_COLOR)}menu{fg(self.terminal.TEXT_COLOR)} - To exit to menu. Example: {fg(self.terminal.HIGHLIGHT_COLOR)}menu{fg(self.terminal.TEXT_COLOR)}")
        self.terminal.print("")
        while True:
            # FIXME: This is ugly
            try:
                self.terminal.print("cwd: ", f"{fg(self.terminal.HIGHLIGHT_COLOR)}{os.getcwd()}\n")
                user_input = input(f"{fg(self.terminal.TEXT_COLOR)}> ")
                split_text = user_input.split(' ')
                split_text = list(filter(lambda text : len(text) > 0, split_text))

                if len(split_text) == 0: continue

                if split_text[0] == 'ask': self.ask(user_input[3:].strip())
                elif split_text[0] == 'menu': break
                elif split_text[0] == 'translate':
                    user_input.replace(split_text[0], '')
                    user_input.replace(split_text[1], '')
                    self.translate(user_input[10:].strip(), split_text[1])
                elif split_text[0] == 'cd': 
                    try: os.chdir(user_input[2:].strip())
                    except: self.terminal.print(f"The system cannot find the file specified: {user_input[2:].strip()}")
                else: os.system(user_input)
            except KeyboardInterrupt: raise KeyboardInterrupt
            except Exception as e: 
                logging.error(traceback.format_exc())

    def ask(self, question : str):
        payload = json.dumps({
            "model": CHATGPT_MODEL,
            "messages": [{"role": "user", "content": question}],
            "temperature": 0.7,
            "stream": True
        })
        res = None
        try:
            session = requests.Session()
            res = session.post(CHATGPT_URL, headers={
                "Content-Type": "application/json", 
                "Authorization": f"Bearer {self.user_credentials[API_KEY]}",
                "OpenAI-Organization": self.user_credentials[ORGANIZATION_KEY]},
                data=payload, stream=True)
            
            gpt_response = ""
            for chunk in res.iter_content(chunk_size=512):
                data = chunk.decode('UTF-8')
                data = data.replace('data: ', '')
                data = data.strip()

                split_lines = data.split('\n')
                for line in split_lines:
                    line = line.strip()
                    if line == "": continue

                    if line == "[DONE]": break
                    
                    line = json.loads(line)
                    if "choices" in line.keys(): line = line["choices"]
                    if len(line) > 0: line = line[0]
                    if "delta" in line.keys(): line = line["delta"]
                    if "content" in line.keys(): line = line["content"]
                    else: line = ""
                    gpt_response += line
            
            curr_size = 0
            new_response = ""
            BOX_SIZE = os.get_terminal_size().columns - 35

            split_response = gpt_response.split(" ")
            for char in split_response:

                if curr_size != 0: new_response += " "
                if curr_size + len(char) > BOX_SIZE: 
                    curr_size = 0
                    new_response += "\n"
                
                new_response += char
                curr_size += len(char)

            self.terminal.box_text(f"{fg(self.terminal.HIGHLIGHT_COLOR)}ChatGPT's response:{fg(self.terminal.TEXT_COLOR)}\n{new_response}")
        except: 
            if res.status_code == 401: print("Wrong credentials")

    def translate(self, question : str, final_lang : str):
        self.ask(f"You are Google translate. Translate the following text to {final_lang}: {question}")

    def configure_user_cred(self):
        organization_key = self.terminal.input(f"{fg(self.terminal.TEXT_COLOR)}Organization key", f"{fg(self.terminal.TEXT_COLOR)}Enter your OpenAI {fg(self.terminal.HIGHLIGHT_COLOR)}organization key{fg(self.terminal.TEXT_COLOR)}. You can get that here: {fg(self.terminal.HIGHLIGHT_COLOR)}https://platform.openai.com/account/org-settings{fg(self.terminal.TEXT_COLOR)}")
        api_key = self.terminal.input("API key", f"{fg(self.terminal.TEXT_COLOR)}Enter your OpenAI {fg(self.terminal.HIGHLIGHT_COLOR)}API key{fg(self.terminal.TEXT_COLOR)}. You can get that here: {fg(self.terminal.HIGHLIGHT_COLOR)}https://platform.openai.com/account/api-keys{fg(self.terminal.TEXT_COLOR)}")
        
        self.user_credentials = {ORGANIZATION_KEY: organization_key, API_KEY: api_key}

        cache_login = self.terminal.selection(
            "Store credentials?",
            f"Would you like to {fg(self.terminal.HIGHLIGHT_COLOR)}store{fg(self.terminal.TEXT_COLOR)} your credentials for future use?",
            [f"{fg(self.terminal.TEXT_COLOR)}Yes",
             f"{fg(self.terminal.TEXT_COLOR)}No"])
        
        if cache_login == f"{fg(self.terminal.TEXT_COLOR)}Yes": 
            Storage.store(ORGANIZATION_KEY, organization_key)
            Storage.store(API_KEY, api_key)

        pass

    def start(self):
        self.shell()
